@@ -4,6 +4,7 @@
 import os
 import re
 import sys
+import copy
 from getopt import getopt
 BASE_DIR = os.path.dirname(os.path.abspath(__file__))
 sys.path.append(BASE_DIR)
@@ -13,7 +14,7 @@ import libmd
 import cat as cat
 import json_to_obs as jo
 
-__version__ = 'v1.3.9'
+__version__ = 'v1.4.0'
 
 class TranslateJson(object):
     def __init__(self, js_file, link_path, dir_type, output_md, output_gitbook, output_xmind, \
@@ -31,6 +32,16 @@ class TranslateJson(object):
         self.output_gitbook = output_gitbook
         self.output_xmind = output_xmind
 
+
+        ###################
+        self.base_addr = ''
+        self.group_addr = ''
+        self.bit_num = 32
+        self.tmp_path_dict = {}
+        self.group_reg_dict = {}
+        ###################
+
+
         self.gitbookpath = ''
         self.markdownpath = ''
         self.data = ''
@@ -43,7 +54,7 @@ class TranslateJson(object):
         self.pre_group = ''
         self.all_reg_data = ''
         self.all_reg_tex_data = ''
-        self.all_reg_list = []
+        self.new_all_reg_list = []
         self.reg_data = '# 寄存器详细说明\n\n'
         self.path_addr_dict = {}
         self.max_len = ''
@@ -89,126 +100,7 @@ class TranslateJson(object):
             for i in tmp:
                 path.append("/".join(in_path.split("/")[:i]))
         return path 
-    
-    def enter_group(self, dpath):
-        #print("enter group")
-        leval = 0
-        group_path = self.get_group_path(dpath)
-        #print(group_path)
-        for i in group_path:
-            leval += 1 
-            if i not in self.history_group:
-                if 'Offset' in self.get_delem(self.js_dict, i):
-                    group_step = int(self.get_delem(self.js_dict, i, key='Offset'), 16)
-                    #self.history_addrr += group_step
-                    self.history_addrr = self.history_offset + group_step
-                    if leval >= 2:
-                        self.history_offset = self.history_addrr
-                    else:
-                        self.history_offset = self.history_base + group_step
-                elif 'Step' in self.get_delem(self.js_dict, i):
-                    group_step = int(self.get_delem(self.js_dict, i, key='Step'), 16)
-                    self.history_addrr += group_step
-                    self.history_offset = self.history_addrr
-                if 'depth' in self.get_delem(self.js_dict, i):
-                    depth = self.get_delem(self.js_dict, i, key = 'depth')
-                else:
-                    depth = 1
-                self.history_group[i] = {'depth':depth, 'num': 1, 'start': self.history_offset}
-    
-    def exit_group(self, dpath):
-        #print("exit group")
-        self.pre_group_list = self.get_group_path(self.pre_group + "/group_reg")
-        now_group_list = self.get_group_path(dpath)
-        self.exit_group_list = []
-        self.enter_group_list = []
-        for i in now_group_list:
-            if i in self.pre_group_list:
-                self.pre_group_list.remove(i)
-            else:
-                self.enter_group_list.append(i)
-        self.exit_group_list = self.pre_group_list
-        elen = len(self.exit_group_list)
-        exit_step = 0
-        next_num = 0 
-        for i in range(elen):
-            index = elen - i -1
-            path = self.exit_group_list[index]
-            if i == 0:
-                next_num = self.history_group[path]['depth'] * self.history_group[path]['num']
-            else:
-                next_num = self.history_group[path]['depth'] * \
-                        (self.history_group[path]['num'] + next_num -1)
-        self.history_addrr = self.history_group[path]['start'] + (next_num-1) * (0x0004)
-        self.history_offset = self.history_addrr
-    
-        if len(self.enter_group_list) > 0:
-            self.enter_group(dpath)
-    
-    
-    def get_addr(self, dpath):
-        #print(dpath)
-        base_changed = 0
-        block_path = dpath.split("registers")[0]
-        base_addr = int(self.get_delem(self.js_dict, block_path, key='base_addr'), 16)
-        if self.history_base == -1:
-            self.history_base = base_addr
-            self.history_offset = base_addr
-            self.history_addrr = base_addr
-        if self.history_base != base_addr:
-            base_changed = 1
-            self.history_base = base_addr
-            self.history_offset = base_addr
-            self.history_addrr = base_addr
-    
-        group_path_list = self.get_group_path(dpath)
-        if len(group_path_list) > 0:
-            if group_path_list not in self.group_set:
-                self.group_set.append(group_path_list)
-                for i in group_path_list:
-                    if i in self.history_group:
-                        self.history_group[i]['num'] +=1
-            else:
-                i = group_path_list[-1]
-                if i in self.history_group:
-                    self.history_group[i]['num'] +=1
-        if group_path_list != []:
-            group_path = group_path_list[-1]
-        else:
-            group_path = ''
-        if base_changed:
-            self.pre_group = group_path
-        else:
-            if group_path != self.pre_group:
-                if self.pre_group == '':
-                    self.enter_group(dpath)
-                else:
-                    if group_path.find(self.pre_group) >= 0:
-                        self.enter_group(dpath)
-                    else:
-                        self.exit_group(dpath)
-                self.pre_group = group_path
-        if 'Step' in self.get_delem(self.js_dict, dpath):
-            offset_addr = self.get_delem(self.js_dict, dpath, key='Step')
-            offset_addr = int(offset_addr,16)
-            now_addr =  self.history_addrr + offset_addr
-            self.history_addrr = now_addr
-            return hex(now_addr)
-        if 'Offset' in self.get_delem(self.js_dict, dpath):
-            #print((self.history_offset))
-            #print('history: ', hex(self.history_offset))
-            offset_addr = self.get_delem(self.js_dict, dpath, key='Offset')
-            offset_addr = int(offset_addr,16)
-            #now_addr =  self.history_offset + offset_addr
-            if group_path:
-                now_addr =  self.history_offset + offset_addr
-            else:
-                now_addr =  self.history_base + offset_addr
-            #print('offset: ', hex(offset_addr))
-            self.history_addrr = now_addr
-            #print('now_addr: ', hex(now_addr))
-            return hex(now_addr)
-    
+
     def find_group_path(self, group, root_path):
         group_list = []
         group_list.append(root_path)
@@ -237,6 +129,28 @@ class TranslateJson(object):
                 'group':[],
                 }
         return group_msg
+
+    def get_step_msg(self, dpath):
+        msg_list = ["N","J","M","k","Z","I","D"]
+        reg_list = self.group_reg_dict[dpath]
+        name = reg_list[0]['name']
+        start_addr = int(reg_list[0]['addr'], 16)
+        tmp = name.split('||')
+        step_msg = ''
+        use_flag = 0
+        for i in range(len(tmp)):
+            n = tmp[i]
+            if n.isdigit() and isinstance(eval(n), int):
+                start = int(n)
+                for r in reg_list:
+                    count = int(r['name'].split('||')[i])
+                    if count == start + 1:
+                        end_addr = int(r['addr'], 16)
+                        step_msg += "+" + msg_list[use_flag] + "*" + hex(end_addr - start_addr)
+                        use_flag += 1
+                        break
+        return step_msg
+                    
     
     def get_groupmsg(self, dpath):
         msg_list = ["N","J","M","k","Z","I","D"]
@@ -272,7 +186,7 @@ class TranslateJson(object):
                     next_num = group_dict[path]['depth'] * \
                             (group_dict[path]['num'] + next_num -1)
             step = next_num * (0x0004)
-            step_msg += "+" + msg_list[j] + "*" + hex(step)
+            #step_msg += "+" + msg_list[j] + "*" + hex(step)
             if total_msg == '':
                 group_msg = self.create_groupmsg()
                 total_msg = group_msg
@@ -284,11 +198,9 @@ class TranslateJson(object):
             scope_msg += " " + msg_list[j] + "的范围为0~" + str(group_dict[path]['depth']-1)
             if j == end:
                 break
-        #print(step_msg)
-        #print(scope_msg)
+        step_msg = self.get_step_msg(dpath)
         return step_msg, scope_msg, total_msg
-    
-    
+
     def get_reserve_fields(self, fields, bit_num):
         bit_num = int(bit_num)
         re_map = [[0, bit_num -1]]
@@ -315,22 +227,6 @@ class TranslateJson(object):
                 })
         #print(re_fields)
         return re_fields
-    
-    
-    def show_group(self, lm, group, name, des, st_addr, hex_reset):
-        for i in range(int(group['range'])):
-            new_name = name + "||" + str(i)
-            new_addr = st_addr + i * group['step']
-            if group['group'] != []:
-                for g in group['group']:
-                    self.show_group(lm, g, new_name, des, new_addr, hex_reset)
-            else:
-                self.all_reg_list.append({
-                    'name':new_name,
-                    'addr':self.zero_fill(hex(new_addr)),
-                    'reset':hex_reset,
-                    'des':des,
-                    })
 
     def add_reset(self, reset, index_range, value):
         scope = index_range.strip("[]").split(":")
@@ -379,7 +275,6 @@ class TranslateJson(object):
                 if i_end <= j_end:
                     fields[i], fields[j] = fields[j], fields[i]
 
-    
     def create_md3(self, dpath, lm, bit_num = 32):
         ls = libsvg.LibSVG()
         reset = {}
@@ -419,17 +314,7 @@ class TranslateJson(object):
         for f in fields:
             ls.add_name(table, f['range'], f['name'])
         hex_reset = self.get_reset(reset, bit_num)
-        if is_group:
-            st_addr = int(addr.split("+")[0], 16)
-            self.show_group(lm, group_msg, name ,des, st_addr, hex_reset)
-        else:
-            st_addr = addr
-            self.all_reg_list.append({
-                'name':name,
-                'addr':st_addr,
-                'reset':hex_reset,
-                'des':des,
-                })
+
         #ls.write_down(table)
         ls.indent(table)
         svg_data = ls.svg_transform(table)
@@ -535,7 +420,7 @@ class TranslateJson(object):
                     in_list[i], in_list[j] = in_list[j], in_list[i] 
 
     def create_all_reg_data(self, lm):
-        for i in self.all_reg_list:
+        for i in self.new_all_reg_list:
             self.all_reg_data += lm.md_create_body_line([i['name'], i['addr'], \
                     i['reset'], i['des']], self.output_md)
             self.all_reg_tex_data += lm.tex_create_body_line([i['name'], i['addr'], \
@@ -562,12 +447,197 @@ class TranslateJson(object):
             fc.write(gitlog)
         return gitlog
 
+    def parse_reg(self, in_dict, tpath = ''):
+        # name 
+        name = in_dict['RegName']
+        name = name.upper()
+
+        # des
+        des = ''
+        if 'Description' in in_dict:
+            des = in_dict['Description']
+
+        # reset
+        reset = {}
+        for k,v in enumerate("0"*self.bit_num):
+            reset[k] = v
+        if 'Fields' in in_dict:
+            for f in in_dict['Fields']:
+                self.add_reset(reset, f['range'], f['reset_value'])
+        hex_reset = self.get_reset(reset, self.bit_num)
+
+        # addr
+        if 'Offset' in in_dict:
+            offset = int(in_dict['Offset'], 16)
+            st_addr =  hex(self.base_addr + offset)
+            self.history_addr = self.base_addr + offset
+        elif 'Step' in in_dict:
+            step = int(in_dict['Step'], 16)
+            st_addr =  hex(self.history_addr + step)
+            self.history_addr = self.history_addr + step
+
+        self.new_all_reg_list.append({
+            'name':name,
+            'addr':st_addr,
+            'reset':hex_reset,
+            'des':des,
+            })
+        #print(tpath, name, st_addr)
+        if tpath not in self.tmp_path_dict:
+            self.tmp_path_dict[tpath] = st_addr
+
+    def parse_group_reg(self, in_dict, index = [], tpath = ''):
+        # 组后缀
+        if index != []:
+            d = "||".join(index)
+
+        # name 
+        name = in_dict['RegName'] + "||" + str(d)
+        name = name.upper()
+
+        # des
+        des = ''
+        if 'Description' in in_dict:
+            des = in_dict['Description']
+
+        # reset
+        reset = {}
+        for k,v in enumerate("0"*self.bit_num):
+            reset[k] = v
+        if 'Fields' in in_dict:
+            for f in in_dict['Fields']:
+                self.add_reset(reset, f['range'], f['reset_value'])
+        hex_reset = self.get_reset(reset, self.bit_num)
+
+        # addr
+        if 'Offset' in in_dict:
+            offset = int(in_dict['Offset'], 16)
+            st_addr =  hex(self.group_offset + offset)
+            self.history_addr = self.group_offset + offset
+        elif 'Step' in in_dict:
+            step = int(in_dict['Step'], 16)
+            st_addr =  hex(self.history_addr + step)
+            self.history_addr = self.history_addr + step
+
+        self.new_all_reg_list.append({
+            'name':name,
+            'addr':st_addr,
+            'reset':hex_reset,
+            'des':des,
+            })
+        #print(tpath, name, st_addr)
+        if tpath not in self.group_reg_dict:
+            self.group_reg_dict[tpath] = []
+        self.group_reg_dict[tpath].append({
+            "name":name,
+            "addr":st_addr,
+            })
+
+        if tpath not in self.tmp_path_dict:
+            self.tmp_path_dict[tpath] = st_addr
+
+    def parse_group_group(self, in_dict, index = [], tpath = ''):
+        depth = in_dict['depth']
+        if "Offset" in in_dict:
+            offset = int(in_dict['Offset'], 16)
+            self.group_offset =  self.group_offset + offset
+        elif "Step" in in_dict:
+            step = int(in_dict['Step'], 16)
+            self.group_offset =  self.history_addr + step
+        for d in range(depth):
+            if d >= 1:
+                if 'Offset' in in_dict['group_reg'][0] and \
+                        int(in_dict['group_reg'][0]['Offset'], 16) == 0:
+                    self.group_offset =  self.history_addr + int(self.bit_num/8)
+            for j in range(len(in_dict['group_reg'])):
+                #print(g['RegName'])
+                g = in_dict['group_reg'][j]
+                tmp = copy.deepcopy(index)
+                tmp.append(str(d))
+                tmp_path = copy.deepcopy(tpath)
+                if 'RegName' in g and 'group_reg' not in g:
+                    self.parse_group_reg(g, tmp, tmp_path + '/group_reg/' + str(j))
+                if 'RegName' in g and  'group_reg' in g:
+                    self.parse_group_group(g, tmp, tmp_path + '/group_reg/' + str(j))
+
+    def parse_group(self, in_dict, index = [], tpath = ''):
+        depth = in_dict['depth']
+        if "Offset" in in_dict:
+            offset = int(in_dict['Offset'], 16)
+            self.group_offset =  self.base_addr + offset
+        elif "Step" in in_dict:
+            step = int(in_dict['Step'], 16)
+            self.group_offset =  self.history_addr + step
+        for d in range(depth):
+            if d >= 1:
+                if 'Offset' in in_dict['group_reg'][0] and \
+                        int(in_dict['group_reg'][0]['Offset'], 16) == 0:
+                    self.group_offset =  self.history_addr + int(self.bit_num/8)
+            for j in range(len(in_dict['group_reg'])):
+                g = in_dict['group_reg'][j]
+                #print(g['RegName'])
+                tmp = copy.deepcopy(index)
+                tmp.append(str(d))
+                tmp_path = copy.deepcopy(tpath)
+                if 'RegName' in g and 'group_reg' not in g:
+                    self.parse_group_reg(g, tmp, tmp_path + '/group_reg/' + str(j))
+                if 'RegName' in g and  'group_reg' in g:
+                    self.parse_group_group(g, tmp, tmp_path + '/group_reg/' + str(j))
+
+
+    def parse_dict(self, in_dict, tpath = ''):
+        if 'RegName' in in_dict and 'group_reg' not in in_dict:
+            self.parse_reg(in_dict, tpath)
+            return 0
+        if 'RegName' in in_dict and  'group_reg' in in_dict:
+            self.parse_group(in_dict, [], tpath)
+            return 0
+        if "base_addr" in in_dict:
+            self.base_addr = int(in_dict['base_addr'], 16)
+            self.history_addr = int(in_dict['base_addr'], 16) 
+            self.bit_num = int(in_dict['d_width'])
+        for k, v in in_dict.items():
+            if isinstance(v, (str,int)):
+                pass
+                #print(k,v)
+            else:
+                pass
+                #print(k,type(v))
+            if isinstance(v, list):
+                if not tpath:
+                    tpath = k
+                else:
+                    tpath += '/' +k
+                for j in range(len(v)):
+                    tmp = copy.deepcopy(tpath)
+                    t = v[j]
+                    if isinstance(t, dict):
+                        self.parse_dict(t, tmp+'/'+str(j))
+
+    def get_addr(self, dpath):
+        return self.tmp_path_dict[dpath]
+
+    def relist(self, in_list):
+        max_len = len(in_list[-1]['addr'][2:])
+        new_list = []
+        for i in in_list:
+            if i['name'].startswith('RESERVED'):
+                continue
+            else:
+                i['addr'] = self.zero_fill(i['addr'])
+                new_list.append(i)
+        self.new_all_reg_list = new_list
 
     def main(self):
         file_msg = os.path.basename(self.js_file).split(".json")[0]
         self.file_msg = file_msg
         pp = preprocess.PreProcess()
         self.js_dict = pp.start(self.js_file)
+        self.parse_dict(self.js_dict)
+        #print(self.js_dict)
+        #print(self.new_all_reg_list)
+        #sys.exit()
+
         self.js_dpath = self.get_dpath(self.js_dict)
         lm = libmd.LibMarkdown()
         self.all_reg_data = lm.md_create_title_line(['名称', '地址', '初值', '描述'])
@@ -597,7 +667,7 @@ class TranslateJson(object):
                             bit_num = self.get_delem(self.js_dict, dpath.split('registers')[0], \
                                     key = 'd_width')
                             self.create_md3(self.js_dpath[i], lm, bit_num)
-                    self.relist(self.all_reg_list)
+                    self.relist(self.new_all_reg_list)
                     self.create_all_reg_data(lm)
                     with open(os.path.join(self.gitbookpath, 'all_register.md'), 'w') as fc:
                         fc.write("## 寄存器列表\n\n" + self.all_reg_data)
@@ -641,7 +711,7 @@ class TranslateJson(object):
                             bit_num = self.get_delem(self.js_dict, dpath.split('registers')[0], \
                                     key = 'd_width')
                             self.create_md3(self.js_dpath[i], lm, bit_num)
-                    self.relist(self.all_reg_list)
+                    self.relist(self.new_all_reg_list)
                     self.create_all_reg_data(lm)
                     gitlog = ''
                     try:
@@ -672,7 +742,7 @@ class TranslateJson(object):
                 bit_num = self.get_delem(self.js_dict, i.split('registers')[0], \
                         key = 'd_width')
                 self.create_md3(i, lm, bit_num)
-            self.relist(self.all_reg_list)
+            self.relist(self.new_all_reg_list)
             self.create_all_reg_data(lm)
             self.data = "# 寄存器列表\n" + self.all_reg_data + "\n"\
                     + "# 寄存器详细说明\n" + self.data
